from vai_toolkit.client import *
import json


def download_csvs(user_secret, pipeline_key, to_file, from_date="", to_date="", table_name = '') -> object:
    """This method is used to download dataset for given pipeline and table of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        to_file {string}: It is file path in which store all csvs.
        from_date {string}: It is from date value in 'MM-DD-YYYY' format.
        to_date {string}: It is to date value in 'MM-DD-YYYY' format.
        table_name {string}: It is name of table to download data.

    returns: 

        type: Response message.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        from_date = "09/01/2023" # From Date
        to_date = "09/11/2023" # From Date
        path = "./new2.zip" #Path
        table='Table 2'

        res = data.download_csvs(user_secret, pipeline_key, path, from_date=from_date, to_date=to_date,table_name=table)

        print(res)
        </span>
        </code></pre>
    """

    try:

        headers =create_headers(user_secret, pipeline= pipeline_key)
        payload = json.dumps({
            "functionName": "download_csvs",
            "fromDate": from_date,
            "table":table_name,
            "toDate": to_date
        })  
        response = validate_response_download(payload,headers )
        if(response.status_code == 200) :
            f = open(to_file, 'wb')
            f.write(response.content)
            f.close()
            return "File saved successfully"
        else :
            response = json.loads(response.text)
            if ('message' in response):
                return  (response['message'])
            else :
                return response
    except Exception as e:
        return e
    
def list_labels(user_secret, pipeline_key, table_name = '') -> object:
    """This method is used to get labels for given pipeline and table of user. 

    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        table_name {string}: It is name of table to download data.

    returns: 

        type: List of all labels.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'

        table_name = 'Table 2'
        res = data.list_labels(user_secret, pipeline_key,table_name =table_name  )

        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            'success': True, 
            'details': {
                'Table Name': 'Table 2', 
                'Column Names': ['id', 'Capital Gain'], 
                'Column Types': ['CATEGORY', 'CATEGORY'], 
                'Column Labels': [None, None]
            }
        }
        </span>
        </code></pre>
    """
    
    headers =create_headers(user_secret, pipeline= pipeline_key) 
    payload = json.dumps({ 
            "functionName": "list_labels",
            "table": table_name
        })
    return validate_response(payload,headers)

def set_labels(user_secret, pipeline_key, labels, d_types, table_name = '') -> object:
    """This method is used to set labels for given pipeline and table of user. 
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        labels {object} : It is dictionary containing the keys as the column of provided pipeline and values are Input, Protected and Output of that column as column label.
        d_types {object}: It is dictionary containing the keys as the column of provided pipeline and values are NUMBER, STRING, and CATEGORICAL of that column as column type.
        table_name {string}: It is name of table to download data.

    returns: 

        type: Get response with added labels.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        columnLabels = { "id" : 'Input' ,"Capital Gain" : 'Output'  } # Column Labels
        columnTypes ={ "id" : 'CATEGORY',"Capital Gain" : 'CATEGORY' } # Column Types
        table_name = 'Table 2'

        res = data.set_labels(user_secret, pipeline_key, columnLabels, columnTypes,table_name)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            'tableName': 'dim_item',
            'columnLabels': [None, 'Output', None, None, 'Input', None, None, None, None],
            'columnNames': ['A1', 'DimItemkey', 'RestaurantCustomerName', 'PLU', 'ItemDescription', 'ItemSubcategory', 'ItemCategory', 'ItemReportingCategory', 'ItemFNBCategory'],
            'columnTypes': ['CATEGORY', 'CATEGORY', 'CATEGORY', 'CATEGORY', 'STRING', 'CATEGORY', 'CATEGORY', 'CATEGORY', 'CATEGORY'],
            'pipeline': 'pipeline id',
            '_id': 'Table id'
        }</span></code></pre>
    """
    
    headers =create_headers(user_secret, pipeline= pipeline_key) 
    payload = json.dumps({ 
            "functionName": "set_labels",
                "columnLabels": labels,
                "columnTypes": d_types,
                "table" : table_name
        })
    return validate_response(payload,headers)

def upload_data(user_secret, pipeline_key, data, date, time, new_columns=False, duplicates=False, new_categories=False, table_name = '') -> object:
    """This method is used to upload data for given pipeline and table of user. 
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        data {obj}: It is data object to upload data.
        date {array}: It is list of dates value in 'MM-DD-YYYY' format.
        time {array}: It is list of time value in 'HH:MM:SS' format.
        new_columns {boolean}: It is boolean value for add new columns either 'True' or 'False'.
        duplicates {boolean}: It is boolean value either 'True' or 'False'.
        new_categories {boolean}: It is boolean value for add new categories either 'True' or 'False'.
        table_name {string}: It is name of table.

    returns: 

        type: Uploaded data object in response.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        uploadData = { "id": [11111, 1], "Capital Gain": [10000, 15000] }   # Columns data
        dates = ["09-06-2023", "09-06-2023"]  # Dates
        times = ["11:08:50", "11:09:50" ]  # Times
        table_name =  'Table 2'

        res = data.upload_data(user_secret, pipeline_key, uploadData, dates, times, new_columns = True, duplicates = True,new_categories =  True ,table_name=table_name)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            'success': True, 
            'data': '{
                "dates":["09-06-2023"]
            }', 
            'warning': 'We insert the duplicate data if we found.'}
        </span>
        </code></pre>
    """

    headers =create_headers(user_secret, pipeline= pipeline_key)  
    payload = json.dumps({ 
        "functionName": "upload_data",
        "date": date,
        "time" : time,
        "data" : data,
        "allowNewColumns" : new_columns,
        "allowDuplicate" : duplicates,
        "allowNewCategory" : new_categories,
        "table":table_name
        })   
    return validate_response(payload,headers)

def upload_csv(user_secret, pipeline_key, path, table_name ) -> object:
    """This method is used to upload csv for given pipeline and table of user. 
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        path {string}: It is the path value of csv file.
        table_name {string}: It is name of table in which you want to upload.

    returns: 

        type: Response message.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        table_name = 'Table 1'

        res = data.upload_csv(user_secret, pipeline_key, "./09-11-2023.csv", table_name)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">Dataset uploaded succesfully.
        </span>
        </code></pre>
    """

    headers =create_headers(user_secret, pipeline= pipeline_key)  
    return validate_response_upload('upload_csv' ,path, table_name,headers)

def set_relations(user_secret, pipeline_key, primary_keys_obj, to_table, relation_array) -> object:
    """This method is used to set relations between tables of given pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        primary_keys_obj {string}: It is an object where key represents the name of table, and it's value is considered as primary key column.
        to_table {string}: It is the table name of which you want to set main table to bind other tables.
        relation_array {array}: It is an array consists of relation object, where object is { "from":"From table name",  "toColumn": "Where column of to table want to bind"}.

    returns: 

        type: Get response message

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        primary_keys_obj = {
            "dim_item": "PLU",
            "check": "CheckKey",
            "check_item": "CheckItemKey"
        }
        to_table =  "check_item"
        relation_array =  [
            {
                "from": "dim_item",
                "toColumn": "ItemNumber"
            },
            {
                "from": "check",
                "toColumn": "CheckKey"
            }
        ]
        res = data.set_relations(user_secret, pipeline_key, primary_keys_obj, to_table, relation_array)

        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "message": "Updated successfully."
        }</span>
        </code></pre>
    """

    try:
        headers =create_headers(user_secret,pipeline= pipeline_key)  
        payload = json.dumps({ 
            "functionName": "set_relations",
            "primaryKeys": primary_keys_obj,
            "toTable" : to_table,
            "relations" : relation_array,
        })   
        return validate_response(payload,headers)
    except Exception as e:
        return e
    

def delete_data(user_secret, pipeline_key, from_date, to_date, table_name = '') -> object:
    """This method is used to delete dataset data for given range in pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        from_date {string}: It is from date value in 'MM-DD-YYYY' format.
        to_date {string}: It is to date value in 'MM-DD-YYYY' format.
        table_name {string}: It is name of table to delete data.

    returns: 

        type: Get response message

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import data

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        from_date = "09/06/2023" # From Date
        to_date = "09/06/2023" # From Date
        table='Table 2'

        res = data.delete_data(user_secret, pipeline_key,  from_date, to_date, table )

        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "message": "Deleted successfully!.."
        }</span>
        </code></pre>
    """
    try:
        headers =create_headers(user_secret,pipeline= pipeline_key)  
        payload = json.dumps({ 
            "functionName": "delete_data",
            "fromDate": from_date,
            "toDate" : to_date,
            "table" : table_name,
        })   
        return validate_response(payload,headers)
    except Exception as e:
        return e


# TODO: finish get all payloads to work ()
# ASSUME: fromDate toDate (both empty means all, from date means all after, to date means all before)

import requests, json 
import time

# url = "https://dev-cloud-api.virtuousai.com/vai-toolkit"

# url_upload = "https://dev-data.virtuousai.com/data/upload_data"

url = "https://cloud-api.virtuousai.com/vai-toolkit"

url_upload = "https://data.virtuousai.com/data/upload_data"


def update_password(user_secret, new_password) -> object:
    """This method is used to update user password.
    
    args:

        user_secret {string}: It is the user secret key.
        new_password {string}: User new password value

    returns:

        type: Response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import client 
        user_secret = 'USER_SECRET_KEY'
        new_password = 'NEW_SECURE_PASSWORD'

        res = client.update_password(user_secret, password)

        print(res)

        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "message": "Password changed successfully!.."
        }</span>
        </code></pre>
        
    """

    headers =create_headers(user_secret)
    if(headers):
        payload =json.dumps({
                "functionName":"update_password", 
                "password" : new_password
        })
        return validate_response(payload, headers=headers)
    else:
         return {"success": False , "message" :"Invalid Connection"}

def update_name(user_secret, first_name, last_name) -> object:
    """This method is used to update user's first name and last name. 
    
    args:

        user_secret {string}: It is the user secret key.
        first_name {string}: User first name value
        last_name {string}: User last name value

    returns:

        type: User object with updated name.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import client 

        user_secret = 'USER_SECRET_KEY'
        first_name = 'John'
        last_name = 'Deo'

        updated_pass = client.update_name(user_secret ,first_name ,last_name )
        print(updated_pass)

        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "email": "john.deo@gmail.com",
            "_id": "61efccd707170700131a87c2",
            "firstName": "John",
            "lastName": "Deo",
        }</span>
        </code></pre>
    """

    try:
        headers =create_headers(user_secret)
        payload =json.dumps({
                "functionName":"update_name", 
                "firstName": first_name, 
                "lastName": last_name
        })
        return validate_response(payload, headers=headers)
    except Exception as e:
        return e

def create_account( email ,password, firstname ,lastname):
    # """CREATE A NEW ORGANIZATION"""
    payload =json.dumps({
            "functionName":"create_account",
             "email" :email ,
             "password" :password , 
             "firstName" :firstname ,
             "lastName" :lastname
        })

    return validate_response(payload, headers={})

def create_organization(user_secret, name, users) -> object:
    """This method is used to create organization for user.
    
    args:

        user_secret {string}: It is the user secret key.
        name {string}: It is name of organization to create
        users {obj}: It is the object consisting of keys as the emails and values as the role to be assigned to that specific email. Eg., { "john.deo@gmail.com" :"ADMIN" }
    
    returns:

        type: Created organization object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import client 

        org_name  = "ORGANIZATION_NAME"
        users = { 'john.deo@gmail.com' : 'ADMIN' }
        user_secret = 'USER_SECRET_KEY'

        res = client.create_organization(user_secret, org_name, users)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "user": "61efccd707170700131a87v2",
            "name": "ORGANIZATION_NAME",
            "datasetS3TotalSize": 0,
            "isDeleted": false,
            "createdAt": "2023-07-19T05:34:13.391Z",
            "updatedAt": "2023-07-19T05:34:13.391Z",
            "_id": "64b775d5f0a90246a2a2c31cd"
        }</span>
        </code></pre>
    """

    headers =create_headers(user_secret)
    payload =json.dumps({
            "functionName":"create_organization",
            "name": name,   
            "users": users, 
        })
    return validate_response(payload, headers=headers)

def check_connection(conn):
    # """CHECKS FOR A VALID CONNECTION"""
    if (len(conn.split(":")) == 2) :
        return {  "user-secret" :conn.split(":")[0],  "pipeline":  conn.split(":")[1]}
    else:
        return False

def create_headers(user , pipeline = ''):
    return {  "user-secret" :user,  "pipeline":pipeline}

def validate_response(payload, headers):
    # """IF 'message', THEN ERROR"""
    response = requests.post(url ,data= payload,headers= headers)
    response = json.loads(response.text)
    if ('message' in response):
        return  (response['message'])
    else :
        return response

def validate_response_download(payload, headers):
    # """IF 'message', THEN ERROR"""
    response = requests.post(url ,data= payload,headers= headers)
    return response

def validate_response_upload(func, file,table ,  headers):
    files = {'file': open(file,'rb')}
    payload ={ 'table_name' : table, 'file_path':int(time.time())  }
    # payload ={ 'table' : table }
    # print(payload)
    response = requests.post(url_upload,data =payload, files=files,headers= headers)
    response = json.loads(response.text)
    if ('message' in response):
        return  (response['message'])
    else :
        return response
   
 


from vai_toolkit.client import *
import json


def create(user_secret, title, type) -> object:
    """This method is used to create pipeline of provided user. 
    
    args:

        user_secret {string}: It is the user secret key.
        title {string}: It is the name of pipeline to assign.
        type {string}: It is the type of pipeline. it could be only 'TABULAR' or 'GRAPH'

    returns: 

        type: Created pipeline response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

        title  = "Tabular data"
        type = "TABULAR"
        user_secret  =  'USER_SECRET_KEY'

        res = pipeline.create(user_secret, title, type)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            '_id': '65017bdc104c04cd9663ba05', 
            'title': 'Tabular data', 
            'apiKey': 'd2159682-aecc-49cf-ba7d-a8bfcc828d5e-1694596060912'
        }</span>
        </code></pre>
    """

    headers =create_headers(user_secret)
    
    payload =json.dumps({
            "functionName": "create_pipeline",
            "title": title,   
            "type": type, 
        })
    
    return validate_response(payload,headers)
     
def list_alerts(user_secret, pipeline_key) -> list:
    """This method is used to get the list of alerts for given pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.

    returns: 

        type: List of all alerts objects.

    Example:
    
        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'

        res = pipeline.list_alerts(user_secret, pipeline_key)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">[
            {
                'timeWindow': {
                    'window': None, 
                    'startTime': None, 
                    'endTime': None, 
                    'column': None, 
                    'dates': [], 
                    'times': [], 
                    'isHoliday': None, 
                    'holidays': [], 
                    'step': None
                    }, 
                    '_id': '65017d26104c04cd9663c23e', 
                    'name': 'test', 
                    'type': 'DATASET', 
                    'pipelineId': '65017bdc104c04cd9663ba05', 
                    'modelId': None, 
                    'creator': '62c56baeb4fbc200139daed0', 
                    'from': '1/1/2023', 
                    'to': '12/31/2023', 
                    'columnFields': ['race', 'hours-per-week', 'age'], 
                    'week': 2, 
                    'thresholdLimit': 49, 
                    'createdAt': '2023-09-13T09:13:10.080Z', 
                    'updatedAt': '2023-09-13T09:13:10.080Z'
            }
        ]
        </span>
        </code></pre>
    """

    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "list_alerts",
        })

    return validate_response(payload,headers)

def create_alert(user_secret, pipeline_key, settings) -> object:
    """This method is used to create alert for given pipeline of user.
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        settings {obj}: It is data object to create alerts.

    returns: 

        type: Created alert response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

        user_secret  =  'USER_SECRET_KEY'
        pipeline_key =  'PIPELINE_KEY'
        data ={
            "name":"test",
            "type":"DATASET",
            "from": "1/1/2023", 
            "to": "12/31/2023",
            "columnFields": ["race",  "hours-per-week", "age"],
            "week": 2,  
            "thresholdLimit": 49
        }

        res = pipeline.create_alert(user_secret, pipeline_key ,data)
        print(res)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            'name': 'test', 
            'type': 'DATASET', 
            'pipelineId': '65017bdc104c04cd9663ba05', 
            'modelId': None, 
            'creator': '62c56baeb4fbc200139daed0', 
            'from': '1/1/2023', 
            'to': '12/31/2023', 
            'columnFields': ['race', 'hours-per-week', 'age'], 
            'week': 2, 
            'thresholdLimit': 49, 
            'timeWindow': {
                'window': None, 
                'startTime': None, 
                'endTime': None, 
                'column': None, 
                'dates': [], 
                'times': [], 
                'isHoliday': None, 
                'holidays': [], 
                'step': None
            }, 
            'createdAt': '2023-09-13T09:13:10.080Z', 
            'updatedAt': '2023-09-13T09:13:10.080Z', 
            '_id': '65017d26104c04cd9663c23e'}
        </span>
        </code></pre>
    """

    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "create_alert",
            "data": settings
        })

    return validate_response(payload,headers)


def create_table(user_secret, pipeline_key, name ) ->object:
    """This method is used to create table for given pipeline of user.

    args:

            user_secret {string}: It is the user secret key.
            pipeline_key {string}: It is the pipeline key.
            name {string}: It is the name of table.

    returns: 

            type: Created alert response object.
    Example:

            <pre class="content-block ml-20"><div class="copy-button">
            <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
            <div class="copy-text"></div>
            </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

            user_secret  =  'USER_SECRET_KEY'
            pipeline_key =  'PIPELINE_KEY'
            name = 'TABLE NAME'

            res = pipeline.create_table(user_secret, pipeline_key ,data)
            print(res)
            </span>
            </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
                'success': True,
                'table': {'name': 'Table Name','pipeline': 'Pipeline id', '_id': 'Table Id'}
            }
        </span>
        </code></pre>
        """
    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "create_table",
            "name": name
        })

    return validate_response(payload,headers)


def update_table(user_secret, pipeline_key,old_name, new_name ) ->str:
    """This method is used to update table name for given pipeline of user.

    args:

            user_secret {string}: It is the user secret key.
            pipeline_key {string}: It is the pipeline key.
            old_name {string}: It is the name of current name of table.
            new_name {string}: It is the new name of table to update.

    returns: 

            type: Created alert response object.
    Example:

            <pre class="content-block ml-20"><div class="copy-button">
            <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
            <div class="copy-text"></div>
            </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

            user_secret  =  'USER_SECRET_KEY'
            pipeline_key =  'PIPELINE_KEY'
            old_name = 'TABLE NAME'
            new_name = 'NEW TABLE NAME'

            res = pipeline.update_table(user_secret, pipeline_key ,old_name, new_name)
            print(res)
            </span>
            </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">Name updated successfully.
        </span>
        </code></pre>
        """
    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "update_table",
            "oldName": old_name,
            "newName": new_name
        })

    return validate_response(payload,headers)

def get_table(user_secret, pipeline_key )  ->object:
    """This method is used to get tables of given pipeline of user.

    args:

            user_secret {string}: It is the user secret key.
            pipeline_key {string}: It is the pipeline key.

    returns: 

            type: Created alert response object.
    Example:

            <pre class="content-block ml-20"><div class="copy-button">
            <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
            <div class="copy-text"></div>
            </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

            user_secret  =  'USER_SECRET_KEY'
            pipeline_key =  'PIPELINE_KEY'

            res = pipeline.get_table(user_secret, pipeline_key )
            print(res)
            </span>
            </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">[
            {
                name : 'TABLE NAME',
                _id : 'Table id',
                pipeline : 'Pipeline id'
            },
            {
                name : 'TABLE NAME 2',
                _id : 'Table id',
                pipeline : 'Pipeline id'
            },
        ]
        </span>
        </code></pre>
            """
    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "get_table",
        })

    return validate_response(payload,headers)


def delete_table(user_secret, pipeline_key, name )  ->str:
    """This method is used to delete table for given pipeline of user.

    args:

            user_secret {string}: It is the user secret key.
            pipeline_key {string}: It is the pipeline key.
            name {string}: It is the name of table.

    returns: 

            type: Created alert response object.
    Example:

            <pre class="content-block ml-20"><div class="copy-button">
            <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
            <div class="copy-text"></div>
            </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

            user_secret  =  'USER_SECRET_KEY'
            pipeline_key =  'PIPELINE_KEY'
            name = 'TABLE NAME TO DELETE'

            res = pipeline.delete_table(user_secret, pipeline_key ,name)
            print(res)
            </span>
            </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">Table deleted successfully.
            
        </span>
        </code></pre>"""
    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "delete_table",
            "name": name
        })

    return validate_response(payload,headers)

def set_default_table(user_secret, pipeline_key, name )  ->str:
    """This method is used to set default table for given pipeline of user.

    args:

            user_secret {string}: It is the user secret key.
            pipeline_key {string}: It is the pipeline key.
            name {string}: It is the name of table.

    returns: 

            type: Created alert response object.
    Example:

            <pre class="content-block ml-20"><div class="copy-button">
            <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
            <div class="copy-text"></div>
            </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import pipeline

            user_secret  =  'USER_SECRET_KEY'
            pipeline_key =  'PIPELINE_KEY'
            name = 'TABLE NAME TO SET DEFAULT'

            res = pipeline.set_default_table(user_secret, pipeline_key ,name)
            print(res)
            </span>
            </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">Default table successfully.
            
        </span>
        </code></pre>"""
    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload = json.dumps({
            "functionName": "set_default_table",
            "name": name
        })

    return validate_response(payload,headers)



from vai_toolkit.client import *
import json



## email group add, get

def get_email_groups(user_secret, pipeline_key) -> list:
    """This method is used to get email list for given pipeline of user.

    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.

    returns:

        type: List of all email groups object.
    
    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'

        res = widget.get_email_groups(user_secret, pipeline_key )
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">[
            {
                "_id": "6204f736f758710015aeceac",
                "name": "EMAIL_GROUP_NAME",
                "users": [
                    "john.doe@gmail.com"
                ]
            }
        ]</span>
        </code></pre>
    """

    headers =create_headers(user_secret,pipeline= pipeline_key)

    payload = json.dumps({
            "functionName" : "get_email_groups",
        })

    return validate_response(payload, headers)


def create_email_groups(user_secret, pipeline_key, group_name, email_list) -> object:
    """This method is used to create email list use in widget for sending the survey.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        group_name {string} : It is the name of group to give.
        email_list {array}: It is the users array consist of object which are having email, firstName, and lastName keys.

    returns: 

        type: Created email group response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        groupName = 'Group 1'
        email_list =  [ { email : "john.deo@gmail.com" ,firstName : "John" , lastName : "Deo"  }]

        res = widget.create_email_groups(user_secret, pipeline_key , groupName, email_list)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "groupName": "Group 1",
            "users": [
                {
                    "email": "john.deo@gmail.com",
                    "firstName": "John",
                    "lastName": "Deo",
                    "isSelected": true,
                    "_id": "64b78349f0a90246a2a2c6b5"
                }
            ],
            "creator": "61efccd707170700131a87c2",
            "isDeleted": false,
            "createdAt": "2023-07-19T06:31:37.465Z",
            "updatedAt": "2023-07-19T06:31:37.465Z",
            "_id": "64b78349f0a90246a2a2c6b4",
            "pipeline": "64b776fbf0a90246a2a2c3c0",
        }</span>
        </code></pre>
    """

    headers =create_headers(user_secret,pipeline= pipeline_key)

    payload = json.dumps({
            "functionName" : "create_email_groups",
            "groupName": group_name,
            "users": email_list ,
        })

    return validate_response(payload, headers)


def create_panel(user_secret, pipeline_key, name):
    headers =create_headers(user_secret,pipeline= pipeline_key)

    payload = json.dumps({
            "functionName" : "create_panel",
            "name": name,
        })

    return validate_response(payload, headers)


## data tracker widget

def create_widget(user_secret, pipeline_key, widget_title, controlPanel, columns) -> object:
    """This method is used to add data tracker widget of timeseries off for given pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        widget_title {string} : It is the name of widget you want to give
        controlPanel {string}: It is the object id of control panel widget of which you want to get prediction.
        columns {array}: It is the list of numeric column fields to which include in widget.

    returns: 

        type: Created widget response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        widget_title = 'WIDGET_TITLE'
        controlPanel = 'PANEL ID'
        columns= ["race", "hours-per-week", "age"]

        res = widget.create_widget(user_secret, pipeline_key , widget_title, controlPanel, columns)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            '_id': '64b7b4e4de9be445d40d7cc8', 
            'widgetName': 'WIDGET_TITLE',
            'creator': '5e996c09e667701bdc9c7546', 
            'pipeline': '64a2c873ff2ea7f10cb83f94',
            'dataSetFields': ['GuestCount']
        }</span>
        </code></pre> 
    """
    
    headers =create_headers(user_secret, pipeline= pipeline_key)

    payload = json.dumps({ 
            "functionName": "create_widget",
            "name": widget_title,
            "controlPanel": controlPanel,
            "dataSetFields": columns
        })

    return validate_response(payload, headers)


def create_timeseries_widget(user_secret, pipeline_key, panel_id, name, type) -> object:
    """This method is used to add data tracker widget of timeseries on for given pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        panel_id {string} : It is the object id of panel which you want to get prediction.
        name {string}: It is the name of widget you want to give.
        type {string}: It is the type of timeseries. It could be only 'WEEK' or 'HOUR'

    returns: 

        type: Created timeseries widget response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        name= 'WIDGET_TITLE'
        panel_id = 'PANEL ID'
        type = 'HOUR'

        res = widget.create_timeseries_widget(user_secret, pipeline_key, panel_id, name, type)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "_id": "64b78462f0a90246a2a2c6eb",
            "creator": "61efccd707170700131a87c2",
            "pipeline": "64b776fbf0a90246a2a2c3c0",
            "name": "WIDGET_TITLE"
        }</span>
        </code></pre>
    """
    
    headers =create_headers(user_secret,pipeline= pipeline_key)
    
    payload =json.dumps({
        "functionName": "create_timeseries",
        "panel" :panel_id,
        "name":name,
        "type" :type
        })

    return validate_response(payload, headers)



## survey widgets
 
def list_survey(user_secret, pipeline_key) -> list:
    """This method is used to get list of survey panel for given pipeline of user. 
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.

    returns: 

        type: List of survey panel objects in provided pipeline and user.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'

        res = widget.list_survey(user_secret, pipeline_key )
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">[
            {
                "_id": "64b7838ef0a90246a2a2c6c6",
                "pipeline": "64b776fbf0a90246a2a2c3c0",
                "name": "Test 1"
            }
        ]</span>
        </code></pre>
    """

    headers =create_headers(user_secret,pipeline= pipeline_key)
        
    payload = json.dumps({ 
        "functionName": "list_survey",
    })    

    return validate_response(payload, headers)


def create_survey_panel(user_secret, pipeline_key, panel_name) -> object:
    """This method is used to add survey panel for given pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        panel_name {string}: It is control panel name

    returns: 

        type: Created survey panel response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        panel_name= 'PANEL_NAME'

        res = widget.create_survey_panel(user_secret, pipeline_key ,panel_name)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "_id": "64b7838ef0a90246a2a2c6c6",
            "creator": "61efccd707170700131a87c2",
            "pipeline": "64b776fbf0a90246a2a2c3c0",
            "panel_name": "PANEL_NAME"
        }</span>
        </code></pre>
    """
    
    headers =create_headers(user_secret, pipeline = pipeline_key)

    payload =json.dumps({
            "functionName": "create_survey_panel",
            "name": panel_name,
        })

    return validate_response(payload, headers)


def create_survey(user_secret, pipeline_key, panel_id, widget_title, questions) -> object:
    """This method is used to add survey question widget for given pipeline of user.
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        panel_id {string} : It is the object id of panel which you want to get prediction.
        widget_title {array}: It is the name of widget you want to give.
        questions {array}: It is array consist of question to and respective answers.

    returns: 

        type: Created survey object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        widget_title= 'WIDGET_TITLE'
        panel_id = 'PANEL ID'
        questions= [
            {
                "queType": "SINGLE_ANSWER",
                "queName": "Name OF QUESTION",
                "options": [
                    {
                        "option": "Option A"
                    },
                    {
                        "option": "Option B"
                    }
                ]
            }
        ]

        res = widget.create_survey(user_secret, pipeline_key , widget_title, panel_id, questions)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "_id": "64b784eaf0a90246a2a2c705",
            "widgetName": "WIDGET_TITLE",
            "creator": "61efccd707170700131a87c2",
            "pipeline": "64b776fbf0a90246a2a2c3c0",
            "customAnalytics": "648171758ae9ce41584b8d02",
            "question": [
                {
                    "queName": "Name OF QUESTION",
                    "queType": "SINGLE_ANSWER",
                    "options": [
                        {
                            "option": "Option A",
                            "lowQueQuality": null,
                            "highQueQuality": null,
                            "_id": "64b784eaf0a90246a2a2c707"
                        },
                        {
                            "option": "Option B",
                            "lowQueQuality": null,
                            "highQueQuality": null,
                            "_id": "64b784eaf0a90246a2a2c708"
                        }
                    ],
                    "isOptionOther": false,
                    "optionDescription": null,
                    "_id": "64b784eaf0a90246a2a2c706"
                }
            ]
        }</span>
        </code></pre> 
    """
    
    headers =create_headers(user_secret,pipeline= pipeline_key)

    payload= json.dumps({
            "functionName": "create_survey",
            "panel" :panel_id,
            "widgetName": widget_title,
            "questions" :questions
        })

    return validate_response(payload, headers)


def update_survey(user_secret, pipeline_key, old_survey_name, new_survey_name) -> object:
    """This method is used to update survey name. 
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        old_survey_name {string} : It is the old name of survey which want to update.
        new_survey_name {array}: It is the new name of survey to update.
        
    returns: 

        type: Updated survey response object.

    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import widget

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        old_survey_name= 'OLD_SURVEY_NAME'
        new_survey_name = 'NEW_SURVEY_NAME'

        res = widget.update_survey(user_secret, pipeline_key ,old_survey_name, new_survey_name)
        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "_id": "64b78422f0a90246a2a2c6df",
            "creator": "61efccd707170700131a87c2",
            "pipeline": "64b776fbf0a90246a2a2c3c0",
            "name": "NEW_SURVEY_NAME"
        }</span>
        </code></pre>
    """

    headers =create_headers(user_secret,pipeline= pipeline_key)

    payload =json.dumps({
        "functionName": "update_survey",
        "surveyName": old_survey_name,
        "newSurveyName": new_survey_name,
    })

    return validate_response(payload, headers)



# TODO: make predict_time and predict_link like "func", "data"

from vai_toolkit.client import *
import json

def train(user_secret, pipeline_key, title,  from_date , to_date, settings, option,table_name='') -> object:
    """This method is used to train model with options "NEXT_SERIES", "MIXED" and "NEXT_LINK", if "NEXT_SERIES" then model will be trained with timeseries on for given pipeline of user and if "MIXED" then model will be trained with timeseries off for given pipeline of user.  
    
    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        title {string}: It is the name of model you want to give
        from_date {string}: It is the date of dataset in pipeline from which model will create in 'MM-DD-YYYY' format, passed if the option is "NEXT_SERIES" and "MIXED" only.
        to_date {string}: It is the date of dataset in pipeline to which model will create in 'MM_DD_YYYY' format, passed if the option is "NEXT_SERIES" and "MIXED" only.
        settings {object}: It is a settings object passed if the option is "NEXT_LINK" only, object with required fields - `dates`, `trainNodeType`, `dummyNodeType`, `seedNodeType` and in `dates` field, add keys as `column_names` with its object values, for Ex. see below example of "NEXT_LINK" option.
        option {string}: It could be only "NEXT_SERIES", "MIXED" and "NEXT_LINK".
        table_name {string}: It is the name of table of which data needs to consider while creating model.

    returns: 

        type: Response message.
    
    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        title = 'MODEL_TITLE'
        from_date = '01/01/2022'
        to_date = '05/30/2023'
        option='NEXT_SERIES' 
        table_name='Table 1'

        res= model.train( user_secret,pipeline_key,title, from_date,to_date, None, option , table_name =table_name)
        print(res)



        # train mixed model

        from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        title = 'MODEL_TITLE'
        from_date = '08/31/2023'
        to_date = '09/11/2023'
        option='MIXED' 
        table_name='Table 1'


        res= model.train( user_secret,pipeline_key,title, from_date,to_date, None,option , table_name =table_name)
        print(res)




        from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        title = 'MODEL_TITLE'

        option  = 'NEXT_LINK'
        settings  = {
            "dates": {
                "dim_item": {
                    "from": "01-01-2023",
                    "to": "12-31-2023",
                    "allDate": False
                },
                "check": {
                    "from": "01-01-2023",
                    "to": "12-31-2023",
                    "allDate": False 
                },
                "check_item": {
                    "from": "01-01-2023",
                    "to": "12-31-2023",
                    "allDate": False
                }
            },
            "trainNodeType": "check",
            "dummyNodeType": "check",
            "seedNodeType": "dim_item"
        }

        res= model.train( user_secret,pipeline_key,title,None,None,settings,option)
        print(res)
        </span>
        </code></pre>


    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "status": "In progress", 
            "model Id": "Model Id", 
            "apiKey": "Model api key."
        }</span>
        </code></pre>

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "status": "In progress", 
            "model Id": "Model Id", 
            "apiKey": "Model api key."
        }</span>
        </code></pre>

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "status": "In progress", 
            "model Id": "Model Id", 
            "apiKey": "Model api key."
        }</span>
        </code></pre>
    
    """

    if(option == "NEXT_SERIES" or option ==  "MIXED" or option == 'NEXT_LINK'):
        headers =create_headers(user_secret, pipeline= pipeline_key)
        payload= {}
        if(option =="NEXT_SERIES" or option ==  "MIXED" ):         
            if table_name:
                payload = json.dumps({ 
                        "functionName":( 'train_timeseries' if option == "NEXT_SERIES" else  ("train_mixed" if option =='MIXED' else 'next_link' )),
                        "title": title,
                        "fromDate": from_date,
                        "toDate": to_date,
                        "tableName":table_name
                    })
            else:
                return { "message" : "Table name is required." }
        else: 
            payload = json.dumps({ 
                "functionName":   'next_link',
                "title" : title,
                "settings": settings
            })
        return validate_response(payload, headers)
    else:
        return {"message":"Only NEXT_SERIES, MIXED and 'NEXT_LINK' as options are allowed in option."}

def predict(user_secret, pipeline_key, model_key, from_date, to_date, config, option) -> object:
    """This method is used to get prediction of table count with different model two options available  "NEXT_SERIES" and "NEXT_LINK".

    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        model_key {string}: It is model key.
        from_date {string}: It is the date of dataset in pipeline from which model will create in 'MM-DD-YYYY' format, passed if the option is "NEXT_SERIES"  only.
        to_date {string}: It is the date of dataset in pipeline to which model will create in 'MM_DD_YYYY' format, passed if the option is "NEXT_SERIES" only.
        config {object}: It is the config object, passed if the option is "NEXT_LINK" only.
        option {string}: It could be only "NEXT_SERIES" or "NEXT_LINK".

    returns: 

        type: Response message.
    
    Example:

        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import model

        user_secret='USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        model_key = 'MODEL_KEY' 

        from_date = '03/01/2023'
        to_date = '05/31/2023'
        option = 'NEXT_SERIES'
        
        res = model.predict(user_secret, pipeline_key, model_key ,from_date,to_date,None,option)
        print(res)



        from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        model_key ='MODEL_KEY'
        config ={
                "seedNodeNameCol" : 'ItemDescription',
                "prediction" : 10,
                "seedNodeNames" : ["1","2"],
                "refNodeDict" : {
                    "GuestCount" :0,
                    "CheckOpen" : "2023-09-08T05:14:05.283Z",
        }}
        option ='NEXT_LINK'

        res = model.predict(user_secret, pipeline_key, model_key, None,None, config, option)
        print(res)

        </span>
        </code></pre>

    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            {
                'data': {
                    'errorMessage': '', 
                    'traceback': '', 
                    'data': {
                        'y': [
                                {'date': '05-31-2023', 'counts': '[{"tableCount":281.7903747559,"time":11},{"tableCount":221.9609527588,"time":12},{"tableCount":136.3621520996,"time":13},{"tableCount":88.3298797607,"time":14}, {"tableCount":66.8720169067,"time":15},{"tableCount":53.803276062,"time":16},{"tableCount":84.1227493286,"time":17},{"tableCount":159.0596923828,"time":18},{"tableCount":186.9671936035,"time":19},{"tableCount":188.2597351074,"time":20},{"tableCount":192.5038146973,"time":21}]'},
                                {'date': '06-01-2023', 'counts': '[{"tableCount":221.7015228271,"time":11},{"tableCount":249.4067230225,"time":12},{"tableCount":262.0877990723,"time":13},{"tableCount":227.6030883789,"time":14},{"tableCount":163.640625,"time":15},{"tableCount":81.578666687,"time":16},{"tableCount":35.5799293518,"time":17},{"tableCount":25.2485866547,"time":18},{"tableCount":16.2483539581,"time":19},{"tableCount":56.0276679993,"time":20},{"tableCount":127.5245361328,"time":21}]'},
                                {'date': '06-02-2023', 'counts': '[{"tableCount":151.8760070801,"time":11},{"tableCount":149.3125305176,"time":12},{"tableCount":136.9319000244,"time":13},{"tableCount":138.5372772217,"time":14},{"tableCount":149.87840271,"time":15},{"tableCount":159.7752838135,"time":16},{"tableCount":138.3939819336,"time":17},{"tableCount":101.8885879517,"time":18},{"tableCount":51.3734474182,"time":19},{"tableCount":26.7129077911,"time":20},{"tableCount":25.0917453766,"time":21}]'}, 
                                {'date': '06-03-2023', 'counts': '[{"tableCount":15.0687656403,"time":11},{"tableCount":40.74168396,"time":12},{"tableCount":89.7492446899,"time":13},{"tableCount":101.6272888184,"time":14},{"tableCount":98.6197280884,"time":15},{"tableCount":94.0878372192,"time":16},{"tableCount":106.0318145752,"time":17},{"tableCount":128.7826690674,"time":18},{"tableCount":150.5857696533,"time":19},{"tableCount":137.3954925537,"time":20},{"tableCount":107.9107666016,"time":21}]'},
                                {'date': '06-04-2023', 'counts': '[{"tableCount":69.9748687744,"time":11},{"tableCount":48.4622573853,"time":12},{"tableCount":38.2721862793,"time":13},{"tableCount":23.1030216217,"time":14},{"tableCount":40.9192276001,"time":15},{"tableCount":80.6420669556,"time":16},{"tableCount":88.2661819458,"time":17},{"tableCount":88.1396331787,"time":18},{"tableCount":92.166053772,"time":19},{"tableCount":116.6257400513,"time":20},{"tableCount":148.4887084961,"time":21}]'}, 
                                {'date': '06-05-2023', 'counts': '[{"tableCount":169.0198364258,"time":11},{"tableCount":147.4364776611,"time":12},{"tableCount":108.4352493286,"time":13},{"tableCount":58.7132301331,"time":14},{"tableCount":31.183631897,"time":15},{"tableCount":28.5562381744,"time":16},{"tableCount":17.1293258667,"time":17},{"tableCount":44.2518920898,"time":18},{"tableCount":96.2329483032,"time":19},{"tableCount":95.2348632812,"time":20},{"tableCount":85.2216186523,"time":21}]'}, 
                                {'date': '06-06-2023', 'counts': '[{"tableCount":89.2415390015,"time":11},{"tableCount":117.0859832764,"time":12},{"tableCount":144.4945983887,"time":13},{"tableCount":158.9694213867,"time":14},{"tableCount":136.0721588135,"time":15},{"tableCount":91.8300094604,"time":16},{"tableCount":37.7490653992,"time":17},{"tableCount":20.4870605469,"time":18},{"tableCount":29.0067577362,"time":19},{"tableCount":17.4881305695,"time":20},{"tableCount":52.84141922,"time":21}]'}
                            ], 
                        'x': '', 
                        'note': 'predicting without explanation'
                }
            }
        }
        </span>
        </code></pre>
    """
    
    if(option == "NEXT_SERIES"  or option == 'NEXT_LINK'):
        headers =create_headers(user_secret,pipeline= pipeline_key)
        payload= {}
        if(option =="NEXT_SERIES"   ):
            payload = json.dumps({ 
                "functionName":  "predict_time",
                "model": model_key,
                "fromDate" : from_date,
                "toDate" : to_date,
            })
        else: 
            payload = json.dumps({ 
                "functionName":   'predict_next_link',
                "model": model_key,
                "config":config
            })
        return validate_response( payload,headers)
    else:
        return {"message":"Only NEXT_SERIES  as options is allowed in option."}


def explain(user_secret, pipeline_key, model_key, datasetData= None, dataset= None, from_date= None , to_date= None, config = None , option="") -> object:
    """This method is used to get explanability values of specific record for given pipeline of user with two options "NEXT_SERIES" or "MIXED", if "NEXT_SERIES" then get explanability values of specific record with timeseries on for given pipeline of user and if "MIXED" then get explanability values of specific record with timeseries off for given pipeline of user.

    args:

        user_secret {string}: It is the user secret key.
        pipeline_key {string}: It is the pipeline key.
        model_key {string}: It is model key.
    
    Keyword Arguments:

        datasetData {string}: It is dataset data value passed if the option is "MIXED" only.
        dataset {string}: It is dataset value passed if the option is "MIXED" only.
        from_date {string}: It is from date string passed if the option is "NEXT_SERIES" only.
        to_date {string}: It is to date string passed if the option is "NEXT_SERIES" only.
        config {object}: It is the config object if the option is "NEXT_LINK" only..
        option {string}: It could be only "NEXT_SERIES" or "MIXED".

    returns: 

        type: Response message.
    
    Example:
        
        <pre class="content-block ml-20"><div class="copy-button">
        <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path fill="#c9c9c9" d="M15 1H4c-1.1 0-2 .9-2 2v13c0 .55.45 1 1 1s1-.45 1-1V4c0-.55.45-1 1-1h10c.55 0 1-.45 1-1s-.45-1-1-1zm.59 4.59l4.83 4.83c.37.37.58.88.58 1.41V21c0 1.1-.9 2-2 2H7.99C6.89 23 6 22.1 6 21l.01-14c0-1.1.89-2 1.99-2h6.17c.53 0 1.04.21 1.42.59zM15 12h4.5L14 6.5V11c0 .55.45 1 1 1z"/></svg>
        <div class="copy-text"></div>
        </div><code class="hljs vbnet python-guide"><span id="text">from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        model_key ='MODEL_KEY'

        from_date = '03/01/2023'
        to_date = '05/31/2023'
        option='NEXT_SERIES'

        res = model.explain(user_secret, pipeline_key, model_key ,from_date=from_date,to_date=to_date,option=option)
        print(res)




        from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        model_key ='MODEL_KEY'

        datasetData ={
            "hours-per-week": 35,
            "age" : 40,
            "loan-approved":0,
            "capital-gain": 0,
            "capital-loss": 0,
            "education": "HS-grad",
            "education-num": 9,
            "marital-status": "Never-married",
            "native-country": "United-States",
            "occupation": "Transport-moving",
            "race": "Non-white",
            "relationship": "Not-in-family",
            "sex": "Male",
            "workclass": "Private"
        }
        dataset ="08-31-2023"

        res = model.explain(user_secret, pipeline_key,model_key, datasetData=datasetData,dataset=dataset, option="MIXED" )
        print(res)


        
        from vai_toolkit import model

        user_secret = 'USER_SECRET_KEY'
        pipeline_key = 'PIPELINE_KEY'
        model_key ='MODEL_KEY'
        config ={
                "seedNodeNameCol" : 'ItemDescription',
                "prediction" : 10,
                "seedNodeNames" : ["1","2"],
                "refNodeDict" : {
                    "GuestCount" :0,
                    "CheckOpen" : "2023-09-08T05:14:05.283Z",
        }}
        option ='NEXT_LINK'

        res = model.explain(user_secret, pipeline_key, model_key, config=config, option=option)
        print(res)
        </span>
        </code></pre>


    Output:

        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text">{
            "data": {
                "errorMessage": "",
                "traceback": "",
                "data": {
                    "y": [
                        {
                            "date": "01-16-2022",
                            "counts": "[{\"tableCount\":22.3249225616,\"time\":11},{\"tableCount\":15.3308525085,\"time\":12},{\"tableCount\":6.0051803589,\"time\":13},{\"tableCount\":6.4271569252,\"time\":14},{\"tableCount\":3.3630287647,\"time\":15},{\"tableCount\":2.7648005486,\"time\":16},{\"tableCount\":5.4221305847,\"time\":17},{\"tableCount\":21.0307579041,\"time\":18},{\"tableCount\":17.9799308777,\"time\":19},{\"tableCount\":11.0987033844,\"time\":20},{\"tableCount\":7.7918105125,\"time\":21}]"
                        },
                        {
                            "date": "01-17-2022",
                            "counts": "[{\"tableCount\":4.3347301483,\"time\":11},{\"tableCount\":13.7139158249,\"time\":12},{\"tableCount\":22.5580196381,\"time\":13},{\"tableCount\":17.7829666138,\"time\":14},{\"tableCount\":10.4020938873,\"time\":15},{\"tableCount\":2.7406756878,\"time\":16},{\"tableCount\":4.4894299507,\"time\":17},{\"tableCount\":3.9393897057,\"time\":18},{\"tableCount\":2.5602560043,\"time\":19},{\"tableCount\":5.2975172997,\"time\":20},{\"tableCount\":24.4320220947,\"time\":21}]"
                        },
                        {
                            "date": "01-18-2022",
                            "counts": "[{\"tableCount\":25.5556297302,\"time\":11},{\"tableCount\":21.6066265106,\"time\":12},{\"tableCount\":16.9466266632,\"time\":13},{\"tableCount\":13.0571289062,\"time\":14},{\"tableCount\":22.881313324,\"time\":15},{\"tableCount\":34.0851211548,\"time\":16},{\"tableCount\":28.8900508881,\"time\":17},{\"tableCount\":23.1612758636,\"time\":18},{\"tableCount\":13.7236795425,\"time\":19},{\"tableCount\":12.1094703674,\"time\":20},{\"tableCount\":8.7841835022,\"time\":21}]"
                        },
                        {
                            "date": "01-19-2022",
                            "counts": "[{\"tableCount\":6.8754734993,\"time\":11},{\"tableCount\":9.8974742889,\"time\":12},{\"tableCount\":29.4479675293,\"time\":13},{\"tableCount\":33.2332077026,\"time\":14},{\"tableCount\":26.8839187622,\"time\":15},{\"tableCount\":22.5158405304,\"time\":16},{\"tableCount\":21.4588184357,\"time\":17},{\"tableCount\":31.3786964417,\"time\":18},{\"tableCount\":41.555103302,\"time\":19},{\"tableCount\":38.0847129822,\"time\":20},{\"tableCount\":33.7989006042,\"time\":21}]"
                        },
                        {
                            "date": "01-20-2022",
                            "counts": "[{\"tableCount\":21.2980632782,\"time\":11},{\"tableCount\":16.9026260376,\"time\":12},{\"tableCount\":9.3885774612,\"time\":13},{\"tableCount\":5.1352725029,\"time\":14},{\"tableCount\":8.3074045181,\"time\":15},{\"tableCount\":26.5341453552,\"time\":16},{\"tableCount\":32.1234817505,\"time\":17},{\"tableCount\":29.40417099,\"time\":18},{\"tableCount\":25.5267181396,\"time\":19},{\"tableCount\":18.9728393555,\"time\":20},{\"tableCount\":21.9795360565,\"time\":21}]"
                        },
                        {
                            "date": "01-21-2022",
                            "counts": "[{\"tableCount\":26.9185009003,\"time\":11},{\"tableCount\":23.7499847412,\"time\":12},{\"tableCount\":19.7978839874,\"time\":13},{\"tableCount\":10.8758878708,\"time\":14},{\"tableCount\":9.7679300308,\"time\":15},{\"tableCount\":4.2197132111,\"time\":16},{\"tableCount\":0.0849183798,\"time\":17},{\"tableCount\":2.3076190948,\"time\":18},{\"tableCount\":18.9048557281,\"time\":19},{\"tableCount\":22.7209529877,\"time\":20},{\"tableCount\":19.5034141541,\"time\":21}]"
                        },
                        {
                            "date": "01-22-2022",
                            "counts": "[{\"tableCount\":16.1986370087,\"time\":11},{\"tableCount\":9.5400733948,\"time\":12},{\"tableCount\":11.5875263214,\"time\":13},{\"tableCount\":16.3533973694,\"time\":14},{\"tableCount\":13.4710626602,\"time\":15},{\"tableCount\":9.5767259598,\"time\":16},{\"tableCount\":3.3179194927,\"time\":17},{\"tableCount\":6.832379818,\"time\":18},{\"tableCount\":5.7702031136,\"time\":19},{\"tableCount\":3.5545077324,\"time\":20},{\"tableCount\":4.9158325195,\"time\":21}]"
                        }
                    ],
                    "x": "",
                    "note": "predicting without explanation"
                }
            }
        }</span>
        </code></pre>


        <pre class="content-block ml-20"><code class="hljs vbnet rest-guide"><span id="text"> {
            "success": true,
            "output": {
                "loan-approved": {
                    "inputs": {
                        "age": [
                            0.056
                        ],
                        "capital-loss": [
                            0
                        ],
                        "education": [
                            -0.037
                        ],
                        "education-num": [
                            -0.008
                        ],
                        "hours-per-week": [
                            0.113
                        ],
                        "native-country": [
                            -0.006
                        ],
                        "occupation": [
                            -0.019
                        ],
                        "relationship": [
                            0.05
                        ],
                        "workclass": [
                            -0.011
                        ]
                    },
                    "outputs": [
                        0.138
                    ]
                }
            },
            "payload": {
                "model": "dev/models/5e99761f522f057aeb4ed3c4/64412fa37a5a840c036a3fce/64a526df900b514e57881ef5_0.1_100_1000_1_17",
                "x": {
                    "age": {
                        "type": "NUMBER",
                        "values": [
                            48
                        ]
                    },
                    "capital-loss": {
                        "type": "CATEGORY",
                        "values": [
                            0
                        ]
                    },
                    "hours-per-week": {
                        "type": "NUMBER",
                        "values": [
                            60
                        ]
                    },
                    "workclass": {
                        "type": "CATEGORY",
                        "values": [
                            "Self-emp-not-inc"
                        ]
                    },
                    "education": {
                        "type": "CATEGORY",
                        "values": [
                            "HS-grad"
                        ]
                    },
                    "education-num": {
                        "type": "CATEGORY",
                        "values": [
                            9
                        ]
                    },
                    "occupation": {
                        "type": "CATEGORY",
                        "values": [
                            "Craft-repair"
                        ]
                    },
                    "relationship": {
                        "type": "CATEGORY",
                        "values": [
                            "Husband"
                        ]
                    },
                    "native-country": {
                        "type": "CATEGORY",
                        "values": [
                            "England"
                        ]
                    }
                },
                "y": {
                    "loan-approved": {
                        "type": "CATEGORY"
                    }
                },
                "file_paths": [
                    "dev/dataset-files/5e99761f522f057aeb4ed3c4/64412fa37a5a840c036a3fce/64412fd67a5a840c036a40fd/04-14-2023.csv"
                ]
            },
            "response": {
                "StatusCode": 200,
                "ExecutedVersion": "$LATEST",
                "Payload": "{\"errorMessage\": \"\", \"traceback\": \"\", \"data\": {\"loan-approved\": {\"inputs\": {\"age\": [0.056], \"capital-loss\": [0.0], \"education\": [-0.037], \"education-num\": [-0.008], \"hours-per-week\": [0.113], \"native-country\": [-0.006], \"occupation\": [-0.019], \"relationship\": [0.05], \"workclass\": [-0.011]}, \"outputs\": [0.138]}}}"
            }
        }</span>
        </code></pre>
    """

    if(option == "NEXT_SERIES" or option ==  "MIXED" or option == 'NEXT_LINK' ) :
            headers =create_headers(user_secret, pipeline= pipeline_key)    
            payload ={ }
            if(option == "NEXT_SERIES"):
                if(from_date and to_date):
                    payload=json.dumps({ 
                        "functionName" :  "explain_timeseries",
                        "model": model_key,
                        "fromDate":from_date,
                        "toDate": to_date
                    }) 
                else:
                   return {"message":"Please provide fromdate and todate for next series option."}
            if( option ==  "MIXED"):
                if(datasetData and dataset):
                    payload=json.dumps({ 
                        "functionName":  "explain",
                        "datasetData": datasetData,
                        "model": model_key,
                        "dataset": dataset,
                    })
                else:
                   return {"message":"Please provide dataset data and dataset for mixed option."}
            if(option == 'NEXT_LINK'):
                payload = json.dumps({ 
                    "functionName":  'explain_link',
                    "model": model_key,
                    "config":config
                })
            return validate_response( payload,headers)
    else:
        return {"message":"Only NEXT_SERIES and MIXED as options are allowed in option."}
    



import requests
import json
import sys

base_url = "https://dev-cloud-api.virtuousai.com/api/v1"

def bulk_pipelines(file):
    files = {'file': open(file,'rb')}
    print(files )
    response = requests.post(base_url + '/automation-add-accounts', files=files)
    res = json.loads(response.text)
    if ('message' in res):
        return  sys.stderr.write(res['message'])
    else :
        return res
    
def bulk_models(file):
    files = {'file': open(file,'rb')}
    print(files )
    response = requests.post(base_url + '/automation-add-models', files=files)
    res = json.loads(response.text)
    if ('message' in res):
        return  sys.stderr.write(res['message'])
    else :
        return res
    
def bulk_upload(file,isDuplicateAllow= True):
    files = {'file': open(file,'rb')}
    response = requests.post(base_url + '/validate-csv-conf-file', files=files)
    res = json.loads(response.text)
    # return res
    if ('message' in res):
        return  sys.stderr.write(res['message'])
    else :
        stored_files={'file': open(file,'rb')}  
        file_contents = json.loads(stored_files['file'].read())
        request_url = base_url + "/upload-csv?isDuplicateAllow="+("true" if isDuplicateAllow is True else "false")
        for  orgKey,organization in file_contents["organizations"].items():
                for key, pipeline in organization["pipelines"].items():
                   for file in pipeline:
                        headers = {     "pipeline":  res[orgKey][key]["apiKey"], "user-secret" :res["ownerId"]}
                        files = {'file': open(file,'rb')}
                        response = requests.post(request_url+"&path="+file, files=files, headers=headers)
                        response = json.loads(response.text)
                        if ('message' in response):
                            print({ 'success' :False , "message" :  (response['message'])}) 
                        else :
                            print(response)
        
def bulk_accounts(file):
    files = {'file': open(file,'rb')}
    response = requests.post(base_url + '/automation-add-access', files=files)
    res = json.loads(response.text)
    if ('message' in res):
        return  sys.stderr.write(res['message'])
    else :
        return res
    
def rollback(id): 
    response = requests.delete(base_url + '/automation-remove/'+id)
    res = json.loads(response.text)
    if ('message' in res):
        return  sys.stderr.write(res['message'])
    else :
        return res
    

